@@ -29,6 +29,8 @@ typedef struct {
     unsigned quantized : 1;
     /* Do we have a valid dithering level? */
     unsigned have_dithering_level : 1;
+    /* Is this a valid object? */
+    unsigned valid : 1;
 }
 image_quantize_t;
 
@@ -72,50 +74,6 @@ iq_png_t;
 
 #define IQFREE iq->n_mallocs--
 
-/* Free the image, but not the attr. */
-
-static void
-image_quantize_free_image (image_quantize_t * iq)
-{
-    if (iq->image) {
-	liq_image_destroy (iq->image);
-	IQFREE;
-	iq->image = 0;
-    }
-    if (iq->result) {
-	liq_result_destroy (iq->result);
-	IQFREE;
-	iq->result = 0;
-    }
-}
-
-/* Free everything except "iq" itself. */
-
-static void
-image_quantize_free_allocated (image_quantize_t * iq)
-{
-    image_quantize_free_image (iq);
-    if (iq->attr) {
-	liq_attr_destroy (iq->attr);
-	IQFREE;
-	iq->attr = 0;
-    }
-}
-
-/* Free everything and check we have the right number of things left. */
-
-static void
-image_quantize_free (image_quantize_t * iq)
-{
-    image_quantize_free_allocated (iq);
-
-    if (iq->n_mallocs != 0) {
-	warn ("Image::Quantize: There are %d pieces of unfreed memory",
-	      iq->n_mallocs);
-    }
-    Safefree (iq);
-}
-
 /*  _____                         
    | ____|_ __ _ __ ___  _ __ ___ 
    |  _| | '__| '__/ _ \| '__/ __|
@@ -143,6 +101,13 @@ liq_error_stuff[] = {
     {LIQ_BAD_USER_POINTER, "liq hates your pointer"},
 };
 
+#define IQVALID(iq) {					\
+	if (! iq->valid) {				\
+	    croak ("%s:%d: invalid image_quantize_t",	\
+		   __FILE__, __LINE__);			\
+	}						\
+    }
+
 #define N_LIQ_ERRORS (sizeof (liq_error_stuff) / sizeof (struct liq_error_c))
 
 static char * image_quantize_get_liq_error_string (int error)
@@ -166,7 +131,7 @@ image_quantize_error (image_quantize_t * iq, liq_error error,
     char * liq_error_string;
     int used;
 
-    image_quantize_free_image (iq);
+    IQVALID (iq);
 
     liq_error_string = image_quantize_get_liq_error_string (error);
 
@@ -179,7 +144,7 @@ image_quantize_error (image_quantize_t * iq, liq_error error,
     vsnprintf (error_message + used, MAX_ERROR_MSG - used,
 	       msg, args);
     va_end (args);
-    croak (error_message);
+    croak ("%s", error_message);
 }
 
 static void
@@ -187,13 +152,73 @@ image_quantize_perl_error (image_quantize_t * iq, char * msg, ...)
 {
     va_list args;
 
-    image_quantize_free_image (iq);
+    IQVALID (iq);
 
     va_start (args, msg);
     vcroak (msg, & args);
     va_end (args);
 }
 
+/* Call "liq_xyz" and complain if it returns an error. */
+
+#define LIQ_CALL(x) {						\
+	liq_error error;					\
+	error = x;						\
+	if (error != LIQ_OK) {					\
+	    image_quantize_error (iq, error,			\
+				  "%s:%d: calling '%s'",	\
+				  __FILE__, __LINE__, #x);	\
+	}							\
+    }
+
+/* Free the image, but not the attr. */
+
+static void
+image_quantize_free_image (image_quantize_t * iq)
+{
+    IQVALID(iq);
+    if (iq->image) {
+	LIQ_CALL (liq_image_destroy (iq->image));
+	IQFREE;
+	iq->image = 0;
+    }
+    if (iq->result) {
+	LIQ_CALL (liq_result_destroy (iq->result));
+	IQFREE;
+	iq->result = 0;
+    }
+}
+
+/* Free everything except "iq" itself. */
+
+static void
+image_quantize_free_allocated (image_quantize_t * iq)
+{
+    IQVALID(iq);
+    image_quantize_free_image (iq);
+    if (iq->attr) {
+	LIQ_CALL (liq_attr_destroy (iq->attr));
+	IQFREE;
+	iq->attr = 0;
+    }
+}
+
+/* Free everything and check we have the right number of things left. */
+
+static void
+image_quantize_free (image_quantize_t * iq)
+{
+    IQVALID(iq);
+    image_quantize_free_allocated (iq);
+
+    if (iq->n_mallocs != 0) {
+	warn ("Image::Quantize: There are %d pieces of unfreed memory",
+	      iq->n_mallocs);
+    }
+    iq->valid = 0;
+    Safefree (iq);
+}
+
 static void
 image_quantize_log_callback (const liq_attr * attr, const char * message,
 			     void * user_info)
@@ -203,6 +228,7 @@ image_quantize_log_callback (const liq_attr * attr, const char * message,
     image_quantize_t * iq;
 
     iq = user_info;
+    IQVALID(iq);
 
     ENTER;
     SAVETMPS;
@@ -244,6 +270,7 @@ image_quantize_t * image_quantize_new ()
     Newxz (iq, 1, image_quantize_t);
     iq->attr = liq_attr_create ();
     IQGET;
+    iq->valid = 1;
     return iq;
 }
 
@@ -265,8 +292,8 @@ image_quantize_png_data_to_rgba (image_quantize_t * iq)
     liq_error error;
     png_uint_32 width;
     png_uint_32 height;
-    png_bytep image_data;
 
+    IQVALID(iq);
     p = iq->hook;
 
     png_get_IHDR (p->in_png, p->in_info, & width, & height,
@@ -322,6 +349,7 @@ static void
 image_quantize_post_quantize (image_quantize_t * iq)
 {
     int error;
+    IQVALID(iq);
     if (iq->have_dithering_level) {
 	error = liq_set_dithering_level (iq->result, iq->dithering_level);
 	if (error != LIQ_OK) {
@@ -353,23 +381,15 @@ image_quantize_png_quantize (image_quantize_t * iq)
 
     png_color png_colours[256];
 
+    IQVALID(iq);
     /* Check the inputs. */
 
-    error = liq_invalid_attr (iq->attr);
-    if (error != LIQ_OK) {
-	image_quantize_error (iq, error, "liq_invalid_attr");
-    }
-    error = liq_invalid_image (iq->image);
-    if (error != LIQ_OK) {
-	image_quantize_error (iq, error, "liq_invalid_image");
-    }
+    LIQ_CALL (liq_invalid_attr (iq->attr));
+    LIQ_CALL (liq_invalid_image (iq->image));
 
     /* Quantize the buffer. */
 
-    error = liq_quantize_image_2 (iq->attr, iq->image, & iq->result);
-    if (error != LIQ_OK) {
-	image_quantize_error (iq, error, "liq_quantize_image_2 failed");
-    }
+    LIQ_CALL (liq_quantize_image_2 (iq->attr, iq->image, & iq->result));
 
     if (! iq->result) {
 	image_quantize_perl_error (iq, "liq_quantize_image_2 failed but returned no error code");
@@ -392,10 +412,10 @@ image_quantize_png_quantize (image_quantize_t * iq)
 
     /* Set up the PNG output. */
 
-    p = iq->hook;
     png_set_IHDR (p->out_png, p->out_info, iq->width, iq->height,
 		  8, PNG_COLOR_TYPE_PALETTE, PNG_INTERLACE_NONE,
 		  PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);
+
     error = liq_write_remapped_image (iq->result, iq->image, 
 				      p->image_data, buffer_size);
     if (error != LIQ_OK) {
