@@ -44,7 +44,25 @@ static const char *const liq_result_magic = "liq_result";
 static const char *const liq_remapping_result_magic = "liq_remapping_result";
 static const char *const liq_freed_magic = "free";
 
-#define CHECK_STRUCT_TYPE(attr, kind) liq_crash_if_invalid_handle_pointer_given((liq_attr*)attr, kind ## _magic)
+#define CHECK_STRUCT_TYPE(attr, kind) {					\
+	liq_error error;						\
+	error =								\
+	    liq_crash_if_invalid_handle_pointer_given((liq_attr*)attr,	\
+						      kind ## _magic);	\
+	if (error) {						        \
+	    return error;						\
+	}								\
+    }
+
+#define CHECK_STRUCT_TYPE_VALUE(attr, kind, value) {			\
+	liq_error error;						\
+	error =								\
+	    liq_crash_if_invalid_handle_pointer_given((liq_attr*)attr,	\
+						      kind ## _magic);	\
+	if (error) {						        \
+	    return value;						\
+	}								\
+    }
 #define CHECK_USER_POINTER(ptr) liq_crash_if_invalid_pointer_given(ptr)
 
 struct liq_attr
@@ -590,21 +608,21 @@ static void contrast_maps(liq_image *image)
     image->edges = edges;
 }
 
-bool liq_crash_if_invalid_handle_pointer_given(const liq_attr *user_supplied_pointer, const char *const expected_magic_header)
+liq_error 
+liq_crash_if_invalid_handle_pointer_given(const liq_attr *user_supplied_pointer, const char *const expected_magic_header)
 {
     if (!user_supplied_pointer) {
-        return false;
+        return LIQ_NULL_POINTER;
     }
 
     if (user_supplied_pointer->magic_header == liq_freed_magic) {
-        fprintf(stderr, "%s used after being freed", expected_magic_header);
-        // this is not normal error handling, this is programmer error that should crash the program.
-        // program cannot safely continue if memory has been used after it's been freed.
-        // abort() is nasty, but security vulnerability may be worse.
-        abort();
+	return LIQ_FREED_POINTER;
     }
 
-    return user_supplied_pointer->magic_header == expected_magic_header;
+    if (user_supplied_pointer->magic_header != expected_magic_header) {
+	return LIQ_INVALID_POINTER;
+    }
+    return LIQ_OK;
 }
 
 bool liq_crash_if_invalid_pointer_given(void *pointer)
@@ -663,7 +681,7 @@ liq_error liq_invalid_result (liq_result * result)
 
 liq_error liq_set_quality(liq_attr* attr, int minimum, int target)
 {
-    if (!CHECK_STRUCT_TYPE(attr, liq_attr)) return LIQ_INVALID_POINTER;
+    CHECK_STRUCT_TYPE(attr, liq_attr);
     if (target < 0 || target > 100 || target < minimum || minimum < 0) return LIQ_VALUE_OUT_OF_RANGE;
 
     attr->target_mse = quality_to_mse(target);
@@ -673,7 +691,7 @@ liq_error liq_set_quality(liq_attr* attr, int minimum, int target)
 
 liq_error liq_get_quality(liq_attr *attr, int *minimum, int *target)
 {
-    if (!CHECK_STRUCT_TYPE(attr, liq_attr)) return LIQ_INVALID_POINTER;
+    CHECK_STRUCT_TYPE(attr, liq_attr);
     if (!CHECK_USER_POINTER(minimum)) return LIQ_INVALID_POINTER;
     if (!CHECK_USER_POINTER(target)) return LIQ_INVALID_POINTER;
 
@@ -684,16 +702,17 @@ liq_error liq_get_quality(liq_attr *attr, int *minimum, int *target)
 
 liq_error liq_set_max_colors(liq_attr* attr, int colors)
 {
-    if (!CHECK_STRUCT_TYPE(attr, liq_attr)) return LIQ_INVALID_POINTER;
-    if (colors < 2 || colors > 256) return LIQ_VALUE_OUT_OF_RANGE;
-
+    CHECK_STRUCT_TYPE(attr, liq_attr);
+    if (colors < 2 || colors > 256) {
+	return LIQ_VALUE_OUT_OF_RANGE;
+    }
     attr->max_colors = colors;
     return LIQ_OK;
 }
 
 liq_error liq_get_max_colors_2(liq_attr *attr, int * max_colors)
 {
-    if (!CHECK_STRUCT_TYPE(attr, liq_attr)) return LIQ_INVALID_POINTER;
+    CHECK_STRUCT_TYPE(attr, liq_attr);
 
     * max_colors = attr->max_colors;
     return LIQ_OK;
@@ -701,7 +720,7 @@ liq_error liq_get_max_colors_2(liq_attr *attr, int * max_colors)
 
 liq_error liq_set_min_posterization(liq_attr *attr, int bits)
 {
-    if (!CHECK_STRUCT_TYPE(attr, liq_attr)) return LIQ_INVALID_POINTER;
+    CHECK_STRUCT_TYPE(attr, liq_attr);
     if (bits < 0 || bits > 4) return LIQ_VALUE_OUT_OF_RANGE;
 
     attr->min_posterization_output = bits;
@@ -710,7 +729,7 @@ liq_error liq_set_min_posterization(liq_attr *attr, int bits)
 
 liq_error liq_get_min_posterization_2(liq_attr *attr, int *bits)
 {
-    if (!CHECK_STRUCT_TYPE(attr, liq_attr)) return LIQ_INVALID_POINTER;
+    CHECK_STRUCT_TYPE(attr, liq_attr);
     *bits = attr->min_posterization_output;
     return LIQ_OK;
 }
@@ -718,9 +737,11 @@ liq_error liq_get_min_posterization_2(liq_attr *attr, int *bits)
 
 liq_error liq_set_speed(liq_attr* attr, int speed)
 {
-    if (!CHECK_STRUCT_TYPE(attr, liq_attr)) return LIQ_INVALID_POINTER;
-    if (speed < 1 || speed > 10) return LIQ_VALUE_OUT_OF_RANGE;
+    CHECK_STRUCT_TYPE(attr, liq_attr);
 
+    if (speed < 1 || speed > 10) {
+	return LIQ_VALUE_OUT_OF_RANGE;
+    }
     int iterations = MAX(8-speed,0); iterations += iterations * iterations/2;
     attr->voronoi_iterations = iterations;
     attr->voronoi_iteration_limit = 1.0/(double)(1<<(23-speed));
@@ -737,27 +758,27 @@ liq_error liq_set_speed(liq_attr* attr, int speed)
 
 liq_error liq_get_speed_2(liq_attr *attr, int *speed)
 {
-    if (!CHECK_STRUCT_TYPE(attr, liq_attr)) return LIQ_INVALID_POINTER;
+    CHECK_STRUCT_TYPE(attr, liq_attr);
     *speed = attr->speed;
     return LIQ_OK;
 }
 
-static void liq_remapping_result_destroy(liq_remapping_result *result)
+static liq_error liq_remapping_result_destroy(liq_remapping_result *result)
 {
-    if (!CHECK_STRUCT_TYPE(result, liq_remapping_result)) return;
+    CHECK_STRUCT_TYPE(result, liq_remapping_result);
 
     if (result->palette) pam_freecolormap(result->palette);
     if (result->pixels) result->free(result->pixels);
 
     result->magic_header = liq_freed_magic;
     result->free(result);
+    return LIQ_OK;
 }
 
 liq_error liq_set_output_gamma(liq_result* res, double gamma)
 {
-    if (!CHECK_STRUCT_TYPE(res, liq_result)) {
-	return LIQ_INVALID_POINTER;
-    }
+    CHECK_STRUCT_TYPE(res, liq_result);
+
     if (gamma <= 0 || gamma >= 1.0) {
 	return LIQ_VALUE_OUT_OF_RANGE;
     }
@@ -773,9 +794,7 @@ liq_error liq_set_output_gamma(liq_result* res, double gamma)
 
 liq_error liq_set_min_opacity(liq_attr* attr, int min)
 {
-    if (!CHECK_STRUCT_TYPE(attr, liq_attr)) {
-	return LIQ_INVALID_POINTER;
-    }
+    CHECK_STRUCT_TYPE(attr, liq_attr);
     if (min < 0 || min > 255) {
 	return LIQ_VALUE_OUT_OF_RANGE;
     }
@@ -785,39 +804,35 @@ liq_error liq_set_min_opacity(liq_attr* attr, int min)
 
 liq_error liq_get_min_opacity_2(liq_attr *attr, int *min)
 {
-    if (!CHECK_STRUCT_TYPE(attr, liq_attr)) {
-	return LIQ_INVALID_POINTER;
-    }
+    CHECK_STRUCT_TYPE(attr, liq_attr);
+
     *min = MIN(255, 256.0 * attr->min_opaque_val);
     return LIQ_OK;
 }
 
 
-void liq_set_last_index_transparent(liq_attr* attr, int is_last)
+liq_error liq_set_last_index_transparent(liq_attr* attr, int is_last)
 {
-    if (!CHECK_STRUCT_TYPE(attr, liq_attr)) {
-	return;
-    }
+    CHECK_STRUCT_TYPE(attr, liq_attr);
     attr->last_index_transparent = !!is_last;
+    return LIQ_OK;
 }
 
-void liq_set_log_callback(liq_attr *attr, liq_log_callback_function *callback, void* user_info)
+liq_error liq_set_log_callback(liq_attr *attr, liq_log_callback_function *callback, void* user_info)
 {
-    if (!CHECK_STRUCT_TYPE(attr, liq_attr)) {
-	return;
-    }
+    CHECK_STRUCT_TYPE(attr, liq_attr);
     liq_verbose_printf_flush(attr);
     attr->log_callback = callback;
     attr->log_callback_user_info = user_info;
+    return LIQ_OK;
 }
 
-void liq_set_log_flush_callback(liq_attr *attr, liq_log_flush_callback_function *callback, void* user_info)
+liq_error liq_set_log_flush_callback(liq_attr *attr, liq_log_flush_callback_function *callback, void* user_info)
 {
-    if (!CHECK_STRUCT_TYPE(attr, liq_attr)) {
-	return;
-    }
+    CHECK_STRUCT_TYPE(attr, liq_attr);
     attr->log_flush_callback = callback;
     attr->log_flush_callback_user_info = user_info;
+    return LIQ_OK;
 }
 
 liq_attr* liq_attr_create()
@@ -825,28 +840,16 @@ liq_attr* liq_attr_create()
     return liq_attr_create_with_allocator(NULL, NULL);
 }
 
-void liq_attr_destroy(liq_attr *attr)
+liq_error
+liq_attr_destroy(liq_attr *attr)
 {
-    if (!CHECK_STRUCT_TYPE(attr, liq_attr)) {
-        return;
-    }
+    CHECK_STRUCT_TYPE(attr, liq_attr);
 
     liq_verbose_printf_flush(attr);
 
     attr->magic_header = liq_freed_magic;
     attr->free(attr);
-}
-
-liq_attr* liq_attr_copy(liq_attr *orig)
-{
-    if (!CHECK_STRUCT_TYPE(orig, liq_attr)) {
-        return NULL;
-    }
-
-    liq_attr *attr = orig->malloc(sizeof(liq_attr));
-    if (!attr) return NULL;
-    *attr = *orig;
-    return attr;
+    return LIQ_OK;
 }
 
 static void *liq_aligned_malloc(size_t size)
@@ -898,9 +901,7 @@ liq_attr* liq_attr_create_with_allocator(void* (*custom_malloc)(size_t), void (*
 
 static liq_error liq_image_create_internal_2 (liq_attr *attr, rgba_pixel* rows[], liq_image_get_rgba_row_callback *row_callback, void *row_callback_user_info, int width, int height, double gamma, liq_image ** image_ptr)
 {
-    if (!CHECK_STRUCT_TYPE(attr, liq_attr)) {
-	return LIQ_BAD_POINTER;
-    }
+    CHECK_STRUCT_TYPE(attr, liq_attr);
     if (width <= 0 || height <= 0) {
 	return LIQ_NEGATIVE_DIMENSIONS;
     }
@@ -954,7 +955,7 @@ static liq_error liq_image_create_internal_2 (liq_attr *attr, rgba_pixel* rows[]
 
 liq_error liq_image_set_memory_ownership(liq_image *img, int ownership_flags)
 {
-    if (!CHECK_STRUCT_TYPE(img, liq_image)) return LIQ_INVALID_POINTER;
+    CHECK_STRUCT_TYPE(img, liq_image);
     if (!img->rows || !ownership_flags || (ownership_flags & ~(LIQ_OWN_ROWS|LIQ_OWN_PIXELS))) {
         return LIQ_VALUE_OUT_OF_RANGE;
     }
@@ -980,48 +981,6 @@ liq_error liq_image_set_memory_ownership(liq_image *img, int ownership_flags)
     return LIQ_OK;
 }
 
-static liq_image *liq_image_create_internal(liq_attr *attr, rgba_pixel* rows[], liq_image_get_rgba_row_callback *row_callback, void *row_callback_user_info, int width, int height, double gamma)
-{
-    if (!CHECK_STRUCT_TYPE(attr, liq_attr) || width <= 0 || height <= 0 || gamma < 0 || gamma > 1.0) return NULL;
-    if (!rows && !row_callback) return NULL;
-
-    liq_image *img = attr->malloc(sizeof(liq_image));
-    if (!img) return NULL;
-    *img = (liq_image){
-        .magic_header = liq_image_magic,
-        .malloc = attr->malloc,
-        .free = attr->free,
-        .width = width, .height = height,
-        .gamma = gamma ? gamma : 0.45455,
-        .rows = rows,
-        .row_callback = row_callback,
-        .row_callback_user_info = row_callback_user_info,
-        .min_opaque_val = attr->min_opaque_val,
-    };
-
-    if (!rows || attr->min_opaque_val < 1.f) {
-        img->temp_row = attr->malloc(sizeof(img->temp_row[0]) * width * omp_get_max_threads());
-        if (!img->temp_row) return NULL;
-    }
-
-    // if image is huge or converted pixels are not likely to be reused then don't cache converted pixels
-    if (liq_image_should_use_low_memory(img, !img->temp_row && !attr->use_contrast_maps && !attr->use_dither_map)) {
-        verbose_print(attr, "  conserving memory");
-        if (!liq_image_use_low_memory(img)) return NULL;
-    }
-
-    if (img->min_opaque_val < 1.f) {
-        verbose_print(attr, "  Working around IE6 bug by making image less transparent...");
-    }
-
-    return img;
-}
-
-liq_image *liq_image_create_custom(liq_attr *attr, liq_image_get_rgba_row_callback *row_callback, void* user_info, int width, int height, double gamma)
-{
-    return liq_image_create_internal(attr, NULL, row_callback, user_info, width, height, gamma);
-}
-
 liq_error liq_image_create_rgba_rows_2 (liq_attr *attr, void* rows[], int width, int height, double gamma, liq_image ** image_ptr)
 {
     liq_error error;
@@ -1042,41 +1001,6 @@ liq_error liq_image_create_rgba_rows_2 (liq_attr *attr, void* rows[], int width,
     return error;
 }
 
-liq_image *liq_image_create_rgba(liq_attr *attr, void* bitmap, int width, int height, double gamma)
-{
-    int i;
-    if (width <= 0 || height <= 0 || gamma < 0 || gamma > 1.0) return NULL;
-    if (width > INT_MAX/16/height || height > INT_MAX/16/width) return NULL;
-    if (!CHECK_STRUCT_TYPE(attr, liq_attr)) return NULL;
-    if (!CHECK_USER_POINTER(bitmap)) return NULL;
-
-    rgba_pixel *pixels = bitmap;
-    rgba_pixel **rows = attr->malloc(sizeof(rows[0])*height);
-    if (!rows) {
-	return NULL;
-    }
-    for(i=0; i < height; i++) {
-        rows[i] = pixels + width * i;
-    }
-
-    liq_image *image = liq_image_create_internal(attr, rows, NULL, NULL, width, height, gamma);
-    image->free_rows = true;
-    image->free_rows_internal = true;
-    return image;
-}
-
-int liq_image_get_width(const liq_image *input_image)
-{
-    if (!CHECK_STRUCT_TYPE(input_image, liq_image)) return -1;
-    return input_image->width;
-}
-
-int liq_image_get_height(const liq_image *input_image)
-{
-    if (!CHECK_STRUCT_TYPE(input_image, liq_image)) return -1;
-    return input_image->height;
-}
-
 typedef void free_func(void*);
 
 free_func *get_default_free_func(liq_image *img)
@@ -1101,10 +1025,10 @@ static void liq_image_free_rgba_source(liq_image *input_image)
     }
 }
 
-void liq_image_destroy(liq_image *input_image)
+liq_error
+liq_image_destroy(liq_image *input_image)
 {
-    if (!CHECK_STRUCT_TYPE(input_image, liq_image)) return;
-
+    CHECK_STRUCT_TYPE(input_image, liq_image);
     liq_image_free_rgba_source(input_image);
 
     if (input_image->noise) {
@@ -1129,6 +1053,7 @@ void liq_image_destroy(liq_image *input_image)
 
     input_image->magic_header = liq_freed_magic;
     input_image->free(input_image);
+    return LIQ_OK;
 }
 
 /* histogram contains information how many times each color is present
@@ -1205,12 +1130,8 @@ liq_error
 liq_quantize_image_2(liq_attr *attr, liq_image *img,
 		     liq_result ** result_ptr)
 {
-    if (!CHECK_STRUCT_TYPE(attr, liq_attr)) {
-	return LIQ_INVALID_POINTER;
-    }
-    if (!CHECK_STRUCT_TYPE(img, liq_image)) {
-	return LIQ_INVALID_POINTER;
-    }
+    CHECK_STRUCT_TYPE(attr, liq_attr);
+    CHECK_STRUCT_TYPE(img, liq_image);
 
     histogram *hist = get_histogram(img, attr);
     if (!hist) {
@@ -1226,7 +1147,7 @@ liq_quantize_image_2(liq_attr *attr, liq_image *img,
 liq_error
 liq_set_dithering_level(liq_result *res, float dither_level)
 {
-    if (!CHECK_STRUCT_TYPE(res, liq_result)) return LIQ_INVALID_POINTER;
+    CHECK_STRUCT_TYPE(res, liq_result);
 
     if (res->remapping) {
         liq_remapping_result_destroy(res->remapping);
@@ -1240,7 +1161,7 @@ liq_set_dithering_level(liq_result *res, float dither_level)
 
 static liq_remapping_result *liq_remapping_result_create(liq_result *result)
 {
-    if (!CHECK_STRUCT_TYPE(result, liq_result)) return NULL;
+    CHECK_STRUCT_TYPE_VALUE(result, liq_result, 0);
 
     liq_remapping_result *res = result->malloc(sizeof(liq_remapping_result));
     if (!res) return NULL;
@@ -1257,16 +1178,18 @@ static liq_remapping_result *liq_remapping_result_create(liq_result *result)
     return res;
 }
 
-double liq_get_output_gamma(const liq_result *result)
+liq_error liq_get_output_gamma(const liq_result *result, double * gamma)
 {
-    if (!CHECK_STRUCT_TYPE(result, liq_result)) return -1;
+    CHECK_STRUCT_TYPE(result, liq_result);
 
-    return result->gamma;
+    * gamma = result->gamma;
+    return LIQ_OK;
 }
 
-void liq_result_destroy(liq_result *res)
+liq_error
+liq_result_destroy(liq_result *res)
 {
-    if (!CHECK_STRUCT_TYPE(res, liq_result)) return;
+    CHECK_STRUCT_TYPE(res, liq_result);
 
     memset(&res->int_palette, 0, sizeof(liq_palette));
 
@@ -1279,11 +1202,12 @@ void liq_result_destroy(liq_result *res)
 
     res->magic_header = liq_freed_magic;
     res->free(res);
+    return LIQ_OK;
 }
 
 double liq_get_quantization_error(liq_result *result)
 {
-    if (!CHECK_STRUCT_TYPE(result, liq_result)) return -1;
+    CHECK_STRUCT_TYPE_VALUE(result, liq_result, -1);
 
     if (result->palette_error >= 0) {
         return result->palette_error*65536.0/6.0;
@@ -1298,7 +1222,7 @@ double liq_get_quantization_error(liq_result *result)
 
 int liq_get_quantization_quality(liq_result *result)
 {
-    if (!CHECK_STRUCT_TYPE(result, liq_result)) return -1;
+    CHECK_STRUCT_TYPE(result, liq_result);
 
     if (result->palette_error >= 0) {
         return mse_to_quality(result->palette_error);
@@ -1339,7 +1263,7 @@ static void set_rounded_palette(liq_palette *const dest, colormap *const map, co
 
 const liq_palette *liq_get_palette(liq_result *result)
 {
-    if (!CHECK_STRUCT_TYPE(result, liq_result)) return NULL;
+    CHECK_STRUCT_TYPE_VALUE(result, liq_result, 0);
 
     if (result->remapping && result->remapping->int_palette.count) {
         return &result->remapping->int_palette;
@@ -1630,8 +1554,9 @@ static void update_dither_map(unsigned char *const *const row_pointers, liq_imag
 liq_error liq_write_remapped_image_rows(liq_result *quant, liq_image *input_image, unsigned char **row_pointers)
 {
     unsigned i;
-    if (!CHECK_STRUCT_TYPE(quant, liq_result)) return LIQ_INVALID_POINTER;
-    if (!CHECK_STRUCT_TYPE(input_image, liq_image)) return LIQ_INVALID_POINTER;
+    CHECK_STRUCT_TYPE(quant, liq_result);
+    CHECK_STRUCT_TYPE(input_image, liq_image);
+
     for (i=0; i < input_image->height; i++) {
         if (!CHECK_USER_POINTER(row_pointers+i) || !CHECK_USER_POINTER(row_pointers[i])) return LIQ_INVALID_POINTER;
     }
@@ -1685,8 +1610,8 @@ liq_error liq_write_remapped_image_rows(liq_result *quant, liq_image *input_imag
 
 liq_error liq_write_remapped_image(liq_result *result, liq_image *input_image, void *buffer, size_t buffer_size)
 {
-    if (!CHECK_STRUCT_TYPE(result, liq_result)) return LIQ_INVALID_POINTER;
-    if (!CHECK_STRUCT_TYPE(input_image, liq_image)) return LIQ_INVALID_POINTER;
+    CHECK_STRUCT_TYPE(result, liq_result);
+    CHECK_STRUCT_TYPE(input_image, liq_image);
     if (!CHECK_USER_POINTER(buffer)) return LIQ_INVALID_POINTER;
 
     const size_t required_size = input_image->width * input_image->height;
