@@ -59,22 +59,15 @@ liq_error liq_invalid_result (liq_result *);
 liq_attr* liq_attr_create(void);
 liq_attr* liq_attr_create_with_allocator(void* (*malloc)(size_t), void (*free)(void*));
 liq_attr* liq_attr_copy(liq_attr* orig);
-void liq_attr_destroy(liq_attr* attr);
+liq_error liq_attr_destroy(liq_attr* attr);
 
 liq_error liq_set_max_colors(liq_attr* attr, int colors);
-int liq_get_max_colors(liq_attr* attr);
 liq_error liq_set_speed(liq_attr* attr, int speed);
-int liq_get_speed(liq_attr* attr);
 liq_error liq_set_min_opacity(liq_attr* attr, int min);
-int liq_get_min_opacity(liq_attr* attr);
 liq_error liq_set_min_posterization(liq_attr* attr, int bits);
-int liq_get_min_posterization(liq_attr* attr);
 liq_error liq_set_quality(liq_attr* attr, int minimum, int maximum);
 liq_error liq_get_quality(liq_attr* attr, int* minimum, int* maximum);
-void liq_set_last_index_transparent(liq_attr* attr, int is_last);
-
-    /* These _2 functions are for the Perl interface. They return an
-       error and the got value in the final pointer. */
+liq_error liq_set_last_index_transparent(liq_attr* attr, int is_last);
 
 liq_error liq_get_max_colors_2(liq_attr* attr, int * colors);
 liq_error liq_get_speed_2(liq_attr* attr, int * speed);
@@ -83,8 +76,8 @@ liq_error liq_get_min_posterization_2(liq_attr* attr, int *bits);
 
 typedef void liq_log_callback_function(const liq_attr*, const char* message, void* user_info);
 typedef void liq_log_flush_callback_function(const liq_attr*, void* user_info);
-void liq_set_log_callback(liq_attr*, liq_log_callback_function*, void* user_info);
-void liq_set_log_flush_callback(liq_attr*, liq_log_flush_callback_function*, void* user_info);
+liq_error liq_set_log_callback(liq_attr*, liq_log_callback_function*, void* user_info);
+liq_error liq_set_log_flush_callback(liq_attr*, liq_log_flush_callback_function*, void* user_info);
 
 liq_image* liq_image_create_rgba_rows(liq_attr* attr, void* rows[], int width, int height, double gamma);
 liq_image* liq_image_create_rgba(liq_attr* attr, void* bitmap, int width, int height, double gamma);
@@ -99,7 +92,7 @@ liq_image* liq_image_create_custom(liq_attr* attr, liq_image_get_rgba_row_callba
 liq_error liq_image_set_memory_ownership(liq_image* image, int ownership_flags);
 int liq_image_get_width(const liq_image* img);
 int liq_image_get_height(const liq_image* img);
-void liq_image_destroy(liq_image* img);
+liq_error  liq_image_destroy(liq_image* img);
 
 liq_result* liq_quantize_image(liq_attr* options, liq_image* input_image);
 
@@ -109,7 +102,7 @@ liq_error liq_quantize_image_2(liq_attr *options, liq_image *input_image, liq_re
 
 liq_error liq_set_dithering_level(liq_result* res, float dither_level);
 liq_error liq_set_output_gamma(liq_result* res, double gamma);
-double liq_get_output_gamma(const liq_result* result);
+liq_error liq_get_output_gamma(const liq_result* result, double * gamma);
 
 const liq_palette* liq_get_palette(liq_result* result);
 
@@ -119,7 +112,7 @@ liq_error liq_write_remapped_image_rows(liq_result* result, liq_image* input_ima
 double liq_get_quantization_error(liq_result* result);
 int liq_get_quantization_quality(liq_result* result);
 
-void liq_result_destroy(liq_result*);
+liq_error liq_result_destroy(liq_result*);
 
 #ifdef __cplusplus
 }
