@@ -1,9 +1,11 @@
+#include <stdlib.h>
+#include <math.h>
+#include <stdbool.h>
+#include <assert.h>
 
-#include "libimagequant.h"
 #include "pam.h"
 #include "nearest.h"
 #include "mempool.h"
-#include <stdlib.h>
 
 struct color_entry {
     f_pixel color;
@@ -34,8 +36,8 @@ static int find_slow(const f_pixel px, const colormap *map)
 {
     int best=0;
     float bestdiff = colordifference(px, map->palette[0].acolor);
+    unsigned int i;
 
-    int i;
     for(i=1; i < map->colors; i++) {
         float diff = colordifference(px, map->palette[i].acolor);
         if (diff < bestdiff) {
@@ -48,10 +50,12 @@ static int find_slow(const f_pixel px, const colormap *map)
 
 static float distance_from_nearest_other_color(const colormap *map, const unsigned int i)
 {
+    unsigned int j;
     float second_best=MAX_DIFF;
-    int j;
-    for(j=0; j < map->colors; j++) {
-        if (i == j) continue;
+    for (j=0; j < map->colors; j++) {
+        if (i == j) {
+	    continue;
+	}
         float diff = colordifference(map->palette[i].acolor, map->palette[j].acolor);
         if (diff <= second_best) {
             second_best = diff;
@@ -71,9 +75,9 @@ static struct head build_head(f_pixel px, const colormap *map, unsigned int num_
 {
     struct sorttmp colors[map->colors];
     unsigned int colorsused=0;
+    unsigned int i;
 
-    int i;
-    for(i=0; i < map->colors; i++) {
+    for (i=0; i < map->colors; i++) {
         if (skip_index[i]) continue; // colors in skip_index have been eliminated already in previous heads
         colors[colorsused].index = i;
         colors[colorsused].radius = colordifference(px, map->palette[i].acolor);
@@ -118,9 +122,9 @@ static colormap *get_subset_palette(const colormap *map)
     }
 
     unsigned int subset_size = (map->colors+3)/4;
-    colormap *subset_palette = pam_colormap(subset_size);
+    colormap *subset_palette = pam_colormap(subset_size, map->malloc, map->free);
+    unsigned int i;
 
-    int i;
     for(i=0; i < subset_size; i++) {
         subset_palette->palette[i] = map->palette[i];
     }
@@ -128,28 +132,26 @@ static colormap *get_subset_palette(const colormap *map)
     return subset_palette;
 }
 
-LIQ_PRIVATE struct nearest_map *nearest_init(const colormap *map, bool fast)
+ struct nearest_map *nearest_init(const colormap *map, bool fast)
 {
     colormap *subset_palette = get_subset_palette(map);
-
+    unsigned int i;
     const unsigned long mempool_size = sizeof(struct color_entry) * subset_palette->colors * map->colors/5 + (1<<14);
     mempool m = NULL;
-    struct nearest_map *centroids = mempool_create(&m, sizeof(*centroids), mempool_size, malloc, free);
+    struct nearest_map *centroids = mempool_create(&m, sizeof(*centroids), mempool_size, map->malloc, map->free);
     centroids->mempool = m;
 
-    int i;
-    for(i=0; i < map->colors; i++) {
+    for (i=0; i < map->colors; i++) {
         const float dist = distance_from_nearest_other_color(map,i);
         centroids->nearest_other_color_dist[i] = dist / 4.f; // half of squared distance
     }
 
     centroids->map = map;
 
+    unsigned int j;
     unsigned int skipped=0;
     assert(map->colors > 0);
-    bool skip_index[map->colors];
- int j;
- for(j=0; j < map->colors; j++) skip_index[j]=false;
+    bool skip_index[map->colors]; for (j=0; j < map->colors; j++) skip_index[j]=false;
 
 
     const unsigned int num_vantage_points = map->colors > 16 ? MIN(map->colors/4, subset_palette->colors) : 0;
@@ -189,7 +191,7 @@ LIQ_PRIVATE struct nearest_map *nearest_init(const colormap *map, bool fast)
         {.a=1,.5, 1, 0}, {.a=1, 1,.5, 0}, {.a=1, 1, 0, .5},
         {.a=1,.5, 1, 1}, {.a=1, 1,.5, 1}, {.a=1, 1, 1, .5},
     };
-    for(i=0; i < sizeof(extrema)/sizeof(extrema[0]); i++) {
+    for (i=0; i < sizeof(extrema)/sizeof(extrema[0]); i++) {
         skip_index[find_slow(extrema[i], map)]=0;
     }
 
@@ -204,10 +206,10 @@ LIQ_PRIVATE struct nearest_map *nearest_init(const colormap *map, bool fast)
     return centroids;
 }
 
-LIQ_PRIVATE unsigned int nearest_search(const struct nearest_map *centroids, const f_pixel px, int likely_colormap_index, const float min_opaque_val, float *diff)
+ unsigned int nearest_search(const struct nearest_map *centroids, const f_pixel px, int likely_colormap_index, const float min_opaque_val, float *diff)
 {
     const bool iebug = px.a > min_opaque_val;
-
+    unsigned int i;
     const struct head *const heads = centroids->heads;
 
     assert(likely_colormap_index < centroids->map->colors);
@@ -217,13 +219,13 @@ LIQ_PRIVATE unsigned int nearest_search(const struct nearest_map *centroids, con
         return likely_colormap_index;
     }
 
-    int i;
-    for(i=0; /* last head will always be selected */ ; i++) {
+    for (i=0; /* last head will always be selected */ ; i++) {
         float vantage_point_dist = colordifference(px, heads[i].vantage_point);
 
         if (vantage_point_dist <= heads[i].radius) {
             assert(heads[i].num_candidates);
             unsigned int ind=0;
+	    unsigned int j;
             float dist = colordifference(px, heads[i].candidates[0].color);
 
             /* penalty for making holes in IE */
@@ -231,8 +233,7 @@ LIQ_PRIVATE unsigned int nearest_search(const struct nearest_map *centroids, con
                 dist += 1.f/1024.f;
             }
 
-            int j;
-            for(j=1; j < heads[i].num_candidates; j++) {
+            for (j=1; j < heads[i].num_candidates; j++) {
                 float newdist = colordifference(px, heads[i].candidates[j].color);
 
                 /* penalty for making holes in IE */
@@ -251,7 +252,7 @@ LIQ_PRIVATE unsigned int nearest_search(const struct nearest_map *centroids, con
     }
 }
 
-LIQ_PRIVATE void nearest_free(struct nearest_map *centroids)
+ void nearest_free(struct nearest_map *centroids)
 {
     mempool_destroy(centroids->mempool);
 }
